import base64
import hashlib

from Crypto import Random
from Crypto.Cipher import AES


BS = 16
pad = lambda s: s + (BS - len(s) % BS) * chr(BS - len(s) % BS)
unpad = lambda s: s[0:-ord(s[-1])]


class AESCipher(object):

    def __init__(self, key):
        self.key = hashlib.sha256(key).digest()

    def encrypt(self, raw):
        raw = pad(raw)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return base64.b64encode(iv + cipher.encrypt(raw))

    def decrypt(self, enc):
        enc = base64.b64decode(enc)
        iv = enc[:16]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return unpad(cipher.decrypt(enc[16:]))



from smite.message import Message
from smite.servant import (
    Servant,
    SecureServant,
    SecureServantIdent,
)
from smite.client import (
    Client,
    Rclient,
)
from smite.proxy import Proxy



import time
import threading
import traceback
import logging
import inspect
import uuid
import sys
from types import ModuleType
from threading import Lock

import zmq
import msgpack
from zope.dottedname.resolve import resolve

from smite.aes_cipher import AESCipher
from smite.exceptions import (
    ServantBindError,
    MessageRecvError,
    RoutineStop,
)


DEFAULT_THREADS_NUM = 5

_STATS_TMPL = {
    'received_messages': 0,
    'processed_messages': 0,
    'malicious_messages': 0,
    'exceptions': 0,
}


log = logging.getLogger('smite.servant')


class Servant(object):

    def __init__(self, methods=None, threads_num=DEFAULT_THREADS_NUM):
        self.methods = {}

        if isinstance(methods, (list, tuple)):
            map(self.register_method, methods)

        elif isinstance(methods, dict):
            for name, method in methods.iteritems():
                self.register_method(method, name)

        elif methods is not None:
            raise ValueError('Invalid \'methods\' argument type: {}'
                             .format(type(methods)))

        self.threads_num = threads_num
        self.backend_uri = ('inproc://smite-backend-{}'
                            .format(uuid.uuid1().hex))
        self.stats = {'threads': {}, 'summary': _STATS_TMPL.copy()}
        self._lock = threading.Lock()
        self._default_method = None

    def set_opts(self, threads_num=None):
        self.threads_num = threads_num | self.threads_num

    def register_method(self, method, name=None):
        if not callable(method):
            raise ValueError('{} is not callable'.format(method))

        if name is None:
            if inspect.isfunction(method):
                name = method.__name__
            elif hasattr(method, '__class__'):
                name = method.__class__.__name__

        if name in self.methods:
            raise ValueError('Method named \'{}\' is already registered'
                             .format(name))

        self.methods[name] = method

    def set_default_method(self, method):
        self._default_method = method

    def expose_module(self, module):
        if not isinstance(module, (str, unicode, ModuleType)):
            raise ValueError('Invalid \'module\' argument type: {}'
                             .format(type(module)))

        if isinstance(module, (str, unicode)):
            module = resolve(module)

        module_functions = filter(
            lambda fn: (fn[1].__module__ == module.__name__
                        and not fn[0].startswith('_')),
            inspect.getmembers(module, inspect.isfunction)
        )
        for fname, fn in module_functions:
            self.register_method(fn, '{}.{}'.format(module.__name__, fname))

    def bind(self, bind_uri):
        self.bind_uri = bind_uri
        self._bind()

    def bind_tcp(self, host, port):
        self.bind_uri = 'tcp://{}:{}'.format(host, port)
        self._bind()

    def bind_ipc(self, address):
        self.bind_uri = 'ipc://{}'.format(address)
        self._bind()

    def bind_inproc(self, address):
        self.bind_uri = 'inproc://{}'.format(address)
        self._bind()

    def run(self):
        self._run = True
        threads = []
        for i in xrange(self.threads_num):
            thread = threading.Thread(target=self.routine)
            thread.daemon = True
            thread.start()
            self.stats['threads'][thread.name] = _STATS_TMPL.copy()
            threads.append(thread)

        poll = zmq.Poller()
        poll.register(self.frontend, zmq.POLLIN)
        poll.register(self.backend,  zmq.POLLIN)

        while self._run:
            sockets = dict(poll.poll(1000))

            if self.frontend in sockets:
                if sockets[self.frontend] == zmq.POLLIN:
                    while True:
                        recv = self.frontend.recv()
                        if self.frontend.getsockopt(zmq.RCVMORE):
                            self.backend.send(recv, zmq.SNDMORE)
                        else:
                            self.backend.send(recv)
                            break

            if self.backend in sockets:
                if sockets[self.backend] == zmq.POLLIN:
                    while True:
                        recv = self.backend.recv()
                        if self.backend.getsockopt(zmq.RCVMORE):
                            self.frontend.send(recv, zmq.SNDMORE)
                        else:
                            self.frontend.send(recv)
                            break

        for thread in threads:
            self.backend.send('__break__')

        for thread in threads:
            thread.join()

        self.frontend.close()
        self.backend.close()
        self.ctx.term()

    def stop(self):
        self._run = False
        time.sleep(1)

    def routine(self):
        socket = self.ctx.socket(zmq.DEALER)
        socket.connect(self.backend_uri)

        # wait for variables to be set
        # TODO: it's stupid, change it
        time.sleep(.2)
        thread_stats = self.stats['threads'][threading.current_thread().name]
        summary_stats = self.stats['summary']

        def increment_stat(name):
            with self._lock:
                thread_stats[name] += 1
                summary_stats[name] += 1

        while True:
            try:
                msg_id, msg = self._recv(socket)
            except MessageRecvError:
                increment_stat('received_messages')
                increment_stat('malicious_messages')
                continue
            except RoutineStop:
                break

            increment_stat('received_messages')

            try:
                msg = msgpack.unpackb(msg)
                log.debug('Received message: {}'.format(msg))
            except:
                log.warn('Message unpack failed')
                increment_stat('malicious_messages')
                # do not bother to reply if client sent some trash
                continue

            try:
                method = self.methods.get(msg['_method'])
                if (
                    method is None and
                    hasattr(self._default_method, '__call__')
                ):
                    rep = {
                        '_result': self._default_method(
                            msg['_method'],
                            *msg.get('args', ()),
                            **msg.get('kwargs', {})
                        )
                    }
                elif method is not None:
                    rep = {'_result': method(*msg.get('args', ()),
                                             **msg.get('kwargs', {}))}
                else:
                    raise KeyError
                if msg.get('_noreply') == 1:
                    increment_stat('processed_messages')
                    continue
            except Exception, e:
                increment_stat('exceptions')
                rep = {
                    '_error': 50,
                    '_exc_msg': e.message if hasattr(e, 'message') else '',
                    '_traceback': traceback.format_exc(),
                }
                log.error(rep['_exc_msg'], exc_info=sys.exc_info())
                if msg.get('_noreply') == 1:
                    continue

            rep['_uid'] = msg['_uid']

            log.info('Sending reply: {}'.format(rep))
            msg_id, rep = self._prepare_reply(msg_id, rep)
            for part in msg_id:
                socket.send(part, zmq.SNDMORE)
            socket.send(rep)
            increment_stat('processed_messages')

        socket.close()

    def _bind(self):
        self.ctx = zmq.Context()
        self.frontend = self.ctx.socket(zmq.ROUTER)

        try:
            self.frontend.bind(self.bind_uri)
            log.info('Servant listening at {}'.format(self.bind_uri))
        except zmq.error.ZMQError as e:
            exc = ServantBindError(
                'Cannot bind to {}'
                .format(self.bind_uri, e.strerror)
            )
            log.exception(exc)
            raise exc

        self.backend = self.ctx.socket(zmq.DEALER)
        self.backend.bind(self.backend_uri)

    def _recv(self, socket):
        msg_id = []
        while True:
            recv = socket.recv()
            if recv == '__break__':
                raise RoutineStop()
            if socket.getsockopt(zmq.RCVMORE):
                msg_id.append(recv)
            else:
                msg = recv
                return msg_id, msg

    def _prepare_reply(self, msg_id, rep):
        return msg_id, msgpack.packb(rep)


class SecureServant(Servant):

    def __init__(self, secret_key, methods=None,
                 threads_num=DEFAULT_THREADS_NUM):
        super(SecureServant, self).__init__(methods, threads_num)
        self.cipher = AESCipher(secret_key)

    def _recv(self, socket):
        msg_id, msg = super(SecureServant, self)._recv(socket)
        dec_msg = self.cipher.decrypt(msg)
        return msg_id, dec_msg

    def _prepare_reply(self, msg_id, rep):
        msg_id, rep = super(SecureServant, self)._prepare_reply(msg_id, rep)
        return msg_id, self.cipher.encrypt(rep)


class SecureServantIdent(Servant):

    def __init__(self, get_key_fn, methods=None,
                 threads_num=DEFAULT_THREADS_NUM):
        super(SecureServantIdent, self).__init__(methods, threads_num)
        self._ciphers = {}  # cipher per ident
        self._msg_id_to_ident = {}
        self._get_key_fn = get_key_fn
        self._lock = Lock()

    def _recv(self, socket):
        msg_id, orig_msg = super(SecureServantIdent, self)._recv(socket)
        ident = orig_msg[:32]
        msg = orig_msg[32:]
        if ident not in self._ciphers:
            key = self._get_key_fn(ident)
            if key is None:
                return msg_id, orig_msg
            else:
                self._ciphers[ident] = AESCipher(self._get_key_fn(ident))

        self._lock.acquire()
        self._msg_id_to_ident[self._msg_id_hash(msg_id)] = ident
        self._lock.release()

        dec_msg = self._ciphers[ident].decrypt(msg)
        return msg_id, dec_msg

    def _prepare_reply(self, msg_id, rep):
        msg_id, rep = (
            super(SecureServantIdent, self)
            ._prepare_reply(msg_id, rep)
        )
        self._lock.acquire()
        ident = self._msg_id_to_ident.pop(self._msg_id_hash(msg_id))
        self._lock.release()
        return msg_id, self._ciphers[ident].encrypt(rep)

    def _msg_id_hash(self, msg_id):
        return hash(frozenset(msg_id))



# -*- coding: utf-8 -*-
import logging

import zmq

from smite.exceptions import ProxyBindError


log = logging.getLogger('smite.proxy')


class Proxy(object):
    def __init__(self, pass_host, pass_port):
        self.pass_uri = 'tcp://{}:{}'.format(pass_host, pass_port)

    def bind(self, host, port):
        frontend_uri = 'tcp://{}:{}'.format(host, port)

        self.ctx = zmq.Context()

        self.frontend = self.ctx.socket(zmq.ROUTER)
        try:
            self.frontend.bind(frontend_uri)
            log.debug('proxy listening on {}'.format(frontend_uri))
        except zmq.error.ZMQError as e:
            exc = ProxyBindError(
                'Cannot bind to {} ({})'
                .format(frontend_uri, e.strerror)
            )
            log.exception(exc)
            raise exc

        self.backend = self.ctx.socket(zmq.DEALER)
        try:
            self.backend.connect(self.pass_uri)
            log.debug('proxy forwarding to {}'.format(self.pass_uri))
        except zmq.error.ZMQError as e:
            exc = ProxyBindError(
                'Cannot bind to {} ({})'
                .format(self.pass_uri, e.strerror)
            )
            log.exception(exc)
            raise exc

    def run(self):
        self._run = True

        poll = zmq.Poller()
        poll.register(self.frontend, zmq.POLLIN)
        poll.register(self.backend,  zmq.POLLIN)

        while self._run:
            sockets = dict(poll.poll(1000))
            log.debug(sockets)

            if self.frontend in sockets:
                if sockets[self.frontend] == zmq.POLLIN:
                    while True:
                        recv = self.frontend.recv()
                        if self.frontend.getsockopt(zmq.RCVMORE):
                            self.backend.send(recv, zmq.SNDMORE)
                        else:
                            self.backend.send(recv)
                            break

            if self.backend in sockets:
                if sockets[self.backend] == zmq.POLLIN:
                    while True:
                        recv = self.backend.recv()
                        if self.backend.getsockopt(zmq.RCVMORE):
                            self.frontend.send(recv, zmq.SNDMORE)
                        else:
                            self.frontend.send(recv)
                            break

        self.frontend.close()
        self.backend.close()
        self.ctx.term()

    def stop(self):
        self._run = False



import uuid
import time
import random
from threading import Thread

import zmq

from smite.message import Message
from smite.servant import (
    Servant,
    SecureServant,
    SecureServantIdent,
)
from smite.client import Client
from smite.proxy import Proxy
from smite.exceptions import (
    ClientTimeout,
    MessageException,
)


HOST = '127.0.0.1'
PORT = 3000
CONNECTION_URI = 'tcp://{}:{}'.format(HOST, PORT)


def client_timeout():
    timeout = 3

    client = Client(default_timeout=timeout)
    client.connect(CONNECTION_URI)
    msg = Message('dummy_method')

    raised = False
    start = time.time()
    try:
        client.send(msg)
    except ClientTimeout:
        raised = True

    assert raised

    # it should take around 3 seconds
    assert 2.5 < time.time() - start < 3.5

    # servant should not get this message after start, it's just gone
    class DummyException(Exception):
        pass

    def dummy_method():
        raise DummyException

    servant = Servant({'dummy_method': dummy_method})
    servant.bind_tcp(HOST, PORT)
    # run servant in separate thread and wait 3 seconds for message
    servant_thread = Thread(target=servant.run)
    servant_thread.start()
    time.sleep(3)
    servant.stop()
    servant_thread.join()

    for thread_stats in servant.stats['threads'].values():
        assert thread_stats['exceptions'] == 0
        assert thread_stats['received_messages'] == 0
        assert thread_stats['malicious_messages'] == 0
        assert thread_stats['processed_messages'] == 0

    assert servant.stats['summary']['exceptions'] == 0
    assert servant.stats['summary']['received_messages'] == 0
    assert servant.stats['summary']['malicious_messages'] == 0
    assert servant.stats['summary']['processed_messages'] == 0


def test_noreply_message():
    servant = Servant({'echo': lambda t: t})
    servant.bind_tcp(HOST, PORT)

    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    client = Client()
    client.connect(CONNECTION_URI)
    msg = Message('echo', uuid.uuid1().hex)
    client.send(msg, noreply=True)

    time.sleep(1)
    assert servant.stats['summary']['received_messages'] == 1
    assert servant.stats['summary']['processed_messages'] == 1

    servant.stop()
    servant_thread.join()


def test_multiple_clients():

    def short_echo(text):
        time.sleep(1)
        return text

    def long_echo(text):
        time.sleep(2)
        return text

    servant = Servant({'short_echo': short_echo, 'long_echo': long_echo})
    servant.bind_tcp(HOST, PORT)
    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    class send_msg(object):
        def __init__(self, method_name):
            self.method_name = method_name

        def __call__(self):
            client = Client()
            client.connect(CONNECTION_URI)
            txt = uuid.uuid4().hex
            msg = Message(self.method_name, txt)
            res = client.send(msg)
            assert res == txt

    client_threads = []
    for method_name in ['short_echo', 'long_echo']:
        thread = Thread(target=send_msg(method_name))
        client_threads.append(thread)
        thread.start()

    # long echo takes 2 seconds
    time.sleep(2.5)

    assert servant.stats['summary']['received_messages'] == 2
    assert servant.stats['summary']['processed_messages'] == 2
    assert servant.stats['summary']['exceptions'] == 0

    servant.stop()
    servant_thread.join()

    for client_thread in client_threads:
        client_thread.join()


def test_proxy():
    host = '127.0.0.1'
    proxy_port = '9000'
    servant_port = '9001'

    def echo(text):
        return text

    servant = Servant({'echo': echo})
    servant.bind_tcp(host, servant_port)
    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    proxy = Proxy(host, servant_port)
    proxy.bind(host, proxy_port)
    proxy_thread = Thread(target=proxy.run)
    proxy_thread.start()

    class send_msg(object):
        def __init__(self, method_name):
            self.method_name = method_name

        def __call__(self):
            time.sleep(.3)
            client = Client()
            client.connect_tcp(host, proxy_port)
            txt = uuid.uuid4().hex
            msg = Message(self.method_name, txt)
            res = client.send(msg)
            assert res == txt

    messages_num = 10
    client_threads = []
    for i in xrange(messages_num):
        thread = Thread(target=send_msg('echo'))
        client_threads.append(thread)
        thread.start()

    time.sleep(1)

    assert servant.stats['summary']['received_messages'] == messages_num
    assert servant.stats['summary']['processed_messages'] == messages_num
    assert servant.stats['summary']['exceptions'] == 0

    servant.stop()
    servant_thread.join()
    proxy.stop()
    proxy_thread.join()


def test_exception_response():
    exc_message = 'This is dummy exception message'

    class DummyException(Exception):
        pass

    def raise_dummy_exc():
        raise DummyException(exc_message)

    servant = Servant({'raise_dummy_exc': raise_dummy_exc})
    servant.bind_tcp(HOST, PORT)
    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    client = Client()
    client.connect(CONNECTION_URI)

    raised = False
    try:
        client.send(Message('raise_dummy_exc'))
    except MessageException, e:
        assert e.message == exc_message
        raised = True

    assert raised

    time.sleep(.1)
    assert servant.stats['summary']['received_messages'] == 1
    assert servant.stats['summary']['exceptions'] == 1
    servant.stop()
    servant_thread.join()


def test_encrypted_messaging():
    secret = 'foobar'

    def multipl(num1, num2):
        return num1 * num2

    servant = SecureServant(methods=[multipl], secret_key=secret)
    servant.bind_tcp(HOST, PORT)
    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    client = Client(secret_key=secret)
    client.connect_tcp(HOST, PORT)
    rep = client.send(Message('multipl', 2, 4))
    assert rep == 8

    servant.stop()
    servant_thread.join()


def test_secure_servant_ident():
    secrets = {
        '38dce0e1b46d4ae089c6d425653b57a9': '1234s9v02',
        'cbe74d25f5764dd8955d4ec137b7de4a': '95nx84mxw',
        '663d817ba3bf4fa6a8f4214c78621294': '4n9t7us7l',
    }

    def get_key(ident):
        return secrets[ident]

    def multipl(num1, num2):
        return num1 * num2

    servant = SecureServantIdent(
        get_key_fn=get_key,
        methods=[multipl],
    )
    servant.bind_tcp(HOST, PORT)
    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    def send_messages(secret_key, ident):
        client = Client(secret_key=secret_key, ident=ident)
        client.connect(CONNECTION_URI)
        for i in range(50):
            a = random.randint(1, 10)
            b = random.randint(1, 10)
            expected_result = multipl(a, b)
            rep = client.send(Message('multipl', a, b))
            assert rep == expected_result

    client_threads = []
    for ident, key in secrets.items():
        thread = Thread(target=send_messages, args=(key, ident))
        client_threads.append(thread)
        thread.start()

    time.sleep(1)
    for t in client_threads:
        t.join()

    servant.stop()
    servant_thread.join()


def test_malicious_messages_non_secure():

    def echo(text):
        return text

    servant = Servant([echo])
    servant.bind_tcp(HOST, PORT)
    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    ctx = zmq.Context()

    socket = ctx.socket(zmq.DEALER)
    poll = zmq.Poller()
    poll.register(socket, zmq.POLLIN)
    socket.connect('tcp://{}:{}'.format(HOST, PORT))

    socket.send('foo')

    sockets = dict(poll.poll(2000))
    assert sockets.get(socket) != zmq.POLLIN

    time.sleep(.2)
    assert servant.stats['summary']['received_messages'] == 1
    assert servant.stats['summary']['processed_messages'] == 0
    assert servant.stats['summary']['malicious_messages'] == 1

    servant.stop()
    servant_thread.join()


def test_malicious_messages_secure():
    secret_1 = 'foo'
    secret_2 = 'bar'

    def echo(text):
        return text

    servant = SecureServant(methods=[echo], secret_key=secret_1)
    servant.bind_tcp(HOST, PORT)
    servant_thread = Thread(target=servant.run)
    servant_thread.start()

    client = Client(secret_key=secret_2)
    client.connect(CONNECTION_URI)
    raised = False
    try:
        client.send(Message('multipl', 2, 4), 2)
    except ClientTimeout:
        raised = True

    assert raised

    time.sleep(.2)
    assert servant.stats['summary']['received_messages'] == 1
    assert servant.stats['summary']['processed_messages'] == 0
    assert servant.stats['summary']['malicious_messages'] == 1

    servant.stop()
    servant_thread.join()



class ConnectionError(Exception):
    pass


class ClientTimeout(Exception):
    pass


class ServantBindError(Exception):
    pass


class ProxyBindError(Exception):
    pass


class MessageException(Exception):

    def __init__(self, message, traceback):
        super(MessageException, self).__init__(message)
        self.traceback = traceback


class MessageRecvError(Exception):
    pass


class RoutineStop(Exception):
    pass



import uuid
import logging

import zmq
import msgpack

from smite.message import Message
from smite.aes_cipher import AESCipher
from smite.exceptions import (
    ConnectionError,
    ClientTimeout,
    MessageException,
)

log = logging.getLogger('smite.client')


class Client(object):

    def __init__(self, secret_key=None, ident=None,
                 default_timeout=5):
        self.ident = ident
        self._default_timeout = default_timeout
        self.cipher = AESCipher(secret_key) if secret_key is not None else None
        self._uuid_node = uuid.getnode()

    def connect(self, connection_uri):
        self.connection_uri = connection_uri
        self._connect()

    def connect_tcp(self, host, port):
        self.connection_uri = 'tcp://{}:{}'.format(host, port)
        self._connect()

    def connect_inproc(self, address):
        self.connection_uri = 'inproc://{}'.format(address)
        self._connect()

    def connect_ipc(self, address):
        self.connection_uri = 'ipc://{}'.format(address)
        self._connect()

    def send(self, msg, timeout=None, noreply=False):
        return self._send(msg, timeout=None, noreply=False)

    def _send(self, msg, timeout=None, noreply=False):
        if not isinstance(msg, Message):
            raise TypeError('\'msg\' argument should be type of \'Message\'')
        if timeout is None:
            timeout = self._default_timeout

        msg_d = {
            '_method': msg.method,
            '_uid': uuid.uuid1(self._uuid_node).hex,
        }
        if noreply:
            msg_d['_noreply'] = 1
        if msg.args:
            msg_d['args'] = msg.args
        if msg.kwargs:
            msg_d['kwargs'] = msg.kwargs

        msg = msg_d
        del msg_d

        log.debug('Sending message: {}'.format(msg))

        msg = msgpack.packb(msg)

        if self.cipher is not None:
            msg = self.cipher.encrypt(msg)

        if self.ident is not None:
            msg = '{}{}'.format(self.ident, msg)
        self._socket.send(msg)

        if noreply:
            return True
        else:
            return self._wait_for_reply(timeout)

    def _connect(self):
        self.ctx = zmq.Context()
        self._create_socket()

    def _wait_for_reply(self, timeout):
        sockets = dict(self._poll.poll(timeout * 1000))

        if sockets.get(self._socket) == zmq.POLLIN:
            rep = self._socket.recv()
            if self.cipher is not None:
                rep = self.cipher.decrypt(rep)
            rep = msgpack.unpackb(rep)
            # TODO: check reply uid and raise exc eventually
            if '_error' in rep:
                if rep['_error'] == 50:
                    log.error(rep['_traceback'])
                    raise MessageException(rep['_exc_msg'], rep['_traceback'])
        else:
            log.error('Message timeout ({} sec) reached -> recreating socket'
                      .format(timeout))
            self._socket.setsockopt(zmq.LINGER, 0)
            self._socket.close()
            self._poll.unregister(self._socket)
            self._create_socket()
            raise ClientTimeout

        return rep['_result']

    def _create_socket(self):
        self._socket = self.ctx.socket(zmq.DEALER)
        self._poll = zmq.Poller()
        self._poll.register(self._socket, zmq.POLLIN)
        try:
            self._socket.connect(self.connection_uri)
        except Exception as e:
            raise ConnectionError(
                'Could not connect to: {} ({})'
                .format(self.connection_uri, e.message)
            )


class RClient(Client):

    def send(self, msg_name, *args, **kw):
        msg = Message(msg_name, *args, **kw)
        return self._send(msg)

    def send_noreply(self, msg_name, *args, **kw):
        msg = Message(msg_name, *args, **kw)
        return self._send(msg, noreply=True)



from setuptools import setup


requires = [
    'psutil==1.2.1',
    'pycrypto==2.6.1',
    'msgpack-python==0.4.0',
    'pyzmq==14.0.1',
    'zope.dottedname==4.0.1',
]


setup(
    name='smite',
    packages=['smite'],
    version='0.1a3',
    install_requires=requires,
    author='pmdez',
    author_email='pawel@mewritescode.com',
    url='https://github.com/pmdz/smite',
    zip_safe=False,
    description='Simple messaging library based on ZMQ',
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 3 - Alpha",
    ],
    entry_points={
        'console_scripts': [
            'smite-servant = smite.cli.servant:main',
        ],
    }
)



class Message(object):

    def __init__(self, method, *args, **kwargs):
        self.method = method
        self.args = args
        self.kwargs = kwargs

    def __repr__(self):
        return ('<Message(\'{}\', args={}, kwargs={})>'
                .format(self.method, self.args, self.kwargs))

    def __str__(self):
        return self.__repr__()
